package com.atlassian.elasticsearch.buckler;

import com.atlassian.elasticsearch.buckler.config.AuthConfig;
import com.atlassian.elasticsearch.buckler.config.BucklerConfig;
import com.atlassian.elasticsearch.buckler.security.AuthRateLimiter;
import com.atlassian.elasticsearch.buckler.security.RequestIdentifier;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.elasticsearch.common.util.concurrent.ThreadContext;
import org.elasticsearch.tasks.Task;
import org.elasticsearch.transport.TransportChannel;
import org.elasticsearch.transport.TransportRequest;
import org.elasticsearch.transport.TransportRequestHandler;

class AuthTransportRequestHandler<T extends TransportRequest> implements TransportRequestHandler<T> {

    private static final Logger log = LogManager.getLogger(AuthTransportRequestHandler.class.getName());

    private final AuthRateLimiter authRateLimiter;
    private final ThreadContext threadContext;
    private final BucklerConfig config;
    private final TransportRequestHandler<T> delegate;

    AuthTransportRequestHandler(TransportRequestHandler<T> delegate, BucklerConfig config,
                                AuthRateLimiter authRateLimiter, ThreadContext threadContext) {
        this.delegate = delegate;
        this.config = config;
        this.authRateLimiter = authRateLimiter;
        this.threadContext = threadContext;
    }

    @Override
    public void messageReceived(T request, TransportChannel channel, Task task) throws Exception {
        if (allow(request, task)) {
            delegate.messageReceived(request, channel, task);
        } else {
            throw new IllegalStateException("Authentication error");
        }
    }

    private boolean allow(T request, Task task) {
        String action = task.getAction();

        AuthConfig authConfig = config.getAuthConfig();
        if (!authConfig.isEnabledForTcp()) {
            log.trace("Skipping transport authentication check for action '{}' because transport authentication is " +
                            "disabled", action);
            return true;
        }

        /*
         * A thread context can only be marked as a system context programmatically (i.e. via Elasticsearch internally
         * or via an Elasticsearch plugin), so a request with a system thread context can always be considered safe.
         * Skipping validation of the Authorization headers for a system thread context can be important as certain
         * repeated system actions (like background checks) can hold onto and share the same system thread context.
         * This means out of date Authorization headers may remain cached (with little that can be done from Buckler
         * itself).
         */
        if (threadContext.isSystemContext()) {
            log.trace("Skipping transport authentication check for action '{}' because current thread context is a " +
                            "system context thread", action);
            return true;
        }

        RequestIdentifier requestIdentifier = RequestIdentifier.from(request);

        /*
         * In the event that we don't already trust this IP and we're in a client or global lockout we're going to
         * instantly fail here. This means that we do not check the password. This will protect from timing attacks
         * as there is no difference between a failed request with the correct password and a failed request with
         * an incorrect password.
         */
        if (!authRateLimiter.isRequestAllowed(requestIdentifier)) {
            log.trace("Rejecting transport authentication for action '{}' because rate limiter disallowed the request",
                    task.getAction());
            return false;
        }

        String authorization = threadContext.getHeader("Authorization");
        if (authorization == null) {
            log.trace("Rejecting transport authentication for action '{}' because is has no authorization header",
                    action);
            return false;
        }

        if (authConfig.isAuthorized(authorization)) {
            log.trace("Accepting transport authentication for action '{}'", action);
            return true;
        } else {
            log.trace("Rejecting transport authentication for action '{}' because it is an invalid authorization " +
                            "header",
                    action);
            authRateLimiter.registerFailure(requestIdentifier);
            return false;
        }
    }
}
