package com.atlassian.elasticsearch.buckler.config;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.core.PathUtils;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.text.MessageFormat;

public class BucklerConfig {

    private static final Logger logger = LogManager.getLogger(BucklerConfig.class.getName());

    // volatile to make sure that new values are visible in case the settings get changed
    private volatile AuthConfig authConfig;
    private volatile BruteForceProtectionSettings bruteForceProtectionSettings;
    private Settings settings;
    private volatile TlsConfig tlsConfig;

    public BucklerConfig(Settings nodeSettings, Path configPath) throws IOException {
        Path configDir = getConfigDir(nodeSettings, configPath);
        Path bucklerDir = configDir.resolve("buckler");
        Path config = bucklerDir.resolve("buckler.yml");
        if (Files.exists(config)) {
            this.settings = Settings.builder().loadFromPath(config).build();
        } else {
            logger.warn("Buckler config file missing, tried path: " + config);
            this.settings = Settings.EMPTY;
        }

        boolean server = nodeSettings.getAsBoolean("network.server", Boolean.TRUE);

        this.authConfig = new AuthConfig(settings);
        this.tlsConfig = new TlsConfig(settings, server);
        this.bruteForceProtectionSettings = new BruteForceProtectionSettings(settings);
    }

    /**
     * @return the current configuration for authentication; should not be cached because the configuration can change
     */
    public AuthConfig getAuthConfig() {
        return authConfig;
    }

    /**
     * @return the current configuration for security lockout and timeout period
     */
    public BruteForceProtectionSettings getBruteForceProtectionSettings() {
        return bruteForceProtectionSettings;
    }

    /**
     * @return the current configuration for TLS; should not be cached because the configuration can change
     */
    public TlsConfig getTlsConfig() {
        return tlsConfig;
    }

    static String getRequiredSetting(Settings settings, String key, String description) {
        String value = settings.get(key);
        if (value == null || value.trim().isEmpty()) {
            throw new BucklerConfigException(MessageFormat.format(description + " ({0})", key));
        }
        return value;
    }

    static boolean loadEnabled(Settings settings, String enabledKey, String description) {
        boolean enabled = settings.getAsBoolean(enabledKey, false);
        logger.info(MessageFormat.format(description + " enabled ({0}): {1}", enabledKey, enabled));
        return enabled;
    }

    private static Path getConfigDir(Settings settings, Path configPath) {
        //In our tests we use BucklerPlugin on the client side and config path will be null if a plugin is used on
        // the client side
        if (configPath != null) {
            return configPath;
        }
        if (settings.get("path.conf") != null) {
            return PathUtils.get(settings.get("path.conf"));
        }
        if (settings.get("default.path.conf") != null) {
            return PathUtils.get(settings.get("default.path.conf"));
        }

        if (settings.get("path.home") != null) {
            Path homeFile = PathUtils.get(settings.get("path.home"));
            return homeFile.resolve("config");
        } else if (settings.get("default.path.home") != null) {
            Path homeFile = PathUtils.get(settings.get("default.path.home"));
            return homeFile.resolve("config");
        } else {
            throw new IllegalStateException("path.home is not configured");
        }
    }
}
